/* 
 * File:   alt_config.h
 * Author: cstgeo
 *
 * Created on April 27, 2018, 1:09 PM
 */

#ifndef ALT_CONFIG_H
#  define	ALT_CONFIG_H
#include "config.h"

// I2C ALT Definitions
/* Possible values for ALT_SAD: {0x60} */
#define ALT_SAD		     0x60 /* ( 110 0000) */

//ND Altitude: 224m

void configure_alt();
#define ALT_CTRL_REG1 0x26
#define ALT_STATUS 0x06
float convertPressure(int P_high, int P_mid, float P_low){
    int sign;
    int count;
    float TOTAL;
    if (P_high > 0x7F) {
        sign = 1;
    }
    else {
        sign = 0;
    }
    TOTAL = (long) P_high << 8 | P_mid;
    for (count = 0; count<8; count++) { //convert LSB to decimal
        P_low = P_low/2;
    }
    TOTAL = TOTAL + P_low;
    return TOTAL;
}

float readAlt(void) {
    int check = 0x00;
    int OUT_P_MSB;
    int OUT_P_CSB;
    float OUT_P_LSB;
    float OUT_T_MSB;
    float OUT_T_LSB;
    float newPressure;
    while (check & 0b00001000 > 0){
        startI2C();
        putI2C(ALT_WRITE & 0xFE);
        putI2C(0x00);
        startI2C();
        putI2C(ALT_READ | 0x01);
        check = getI2C(1);
        stopI2C();   
        printf("check = %i \n\r", check);
    }
    startI2C();
    putI2C(ALT_WRITE & 0xFE);
    putI2C(0x01);
    startI2C();
    putI2C(ALT_READ | 0x01);
    OUT_P_MSB = getI2C(0);
    OUT_P_CSB = getI2C(0);
    OUT_P_LSB = getI2C(1);
    stopI2C();
    newPressure = convertPressure(OUT_P_MSB, OUT_P_CSB,OUT_P_LSB);
    return newPressure;
}

void configure_alt() {
    int reg_1_data = 0x00;
    int PT_DATA = 0x00;
    int reg_1_ACTIVE = 0x00;
    while (reg_1_data != 0xB8){
        startI2C();
        putI2C(ALT_WRITE & 0xFE);
        putI2C(ALT_CTRL_REG1);
        putI2C(0xB8);
        stopI2C();
        startI2C();
        putI2C(ALT_WRITE & 0xFE);
        putI2C(ALT_CTRL_REG1);
        startI2C();
        putI2C(ALT_READ |0x01);
        reg_1_data = getI2C(1);
        stopI2C();
    }
    while (PT_DATA != 0x07) {
        startI2C();
        putI2C(ALT_WRITE & 0xFE);
        putI2C(0x13);
        putI2C(0x07);
        stopI2C();
        startI2C();
        putI2C(ALT_WRITE & 0xFE);
        putI2C(0x13);
        startI2C();
        putI2C(ALT_READ | 0x01);
        PT_DATA = getI2C(1);
        stopI2C();
    }
    while (reg_1_data != 0xB9){
        startI2C();
        putI2C(ALT_WRITE & 0xFE);
        putI2C(ALT_CTRL_REG1);
        putI2C(0xB9);
        stopI2C();
        startI2C();
        putI2C(ALT_WRITE & 0xFE);
        putI2C(ALT_CTRL_REG1);
        startI2C();
        putI2C(ALT_READ |0x01);
        reg_1_data = getI2C(1);
        stopI2C();
    }
}


// [OUT_P_MSB, OUT_P_CSB, OUT_P_LSB, OUT_T_MSB, OUT_T_LSB]
sym alt_data[ALT_REG_COUNT+1] = {0};
sym * read_alt(void) {
    int check = 0x00;
    while (check & 0b00001000 > 0){
        startI2C();
        putI2C(ALT_WRITE);
        putI2C(0x00);
        startI2C();
        putI2C(ALT_READ);
        check = getI2C(1);
        stopI2C();   
        printf("check = %i \n\r", check); // haven't seen print
    }
    
		startI2C();
    putI2C(ALT_WRITE);
    putI2C(0x01);
		
    startI2C();
    putI2C(ALT_READ);
		alt_data[0] = getI2C(0) & 0xFF;	// OUT_P_MSB Register @ SUB 0x01  ---  PD[19:12]  // The MSB of the integer part of the altitude value.
		alt_data[1] = getI2C(0) & 0xFF;	// OUT_P_CSB Register @ SUB 0x02  ---  PD[11:4]		// The LSB of the integer part of the altitude value.
		alt_data[2] = getI2C(0) & 0xF0;	// OUT_P_LSB Register @ SUB 0x03  ---  PD[3:0]			// The upper nybble is the fractional part of the altitude value.
		alt_data[3] = getI2C(0) & 0xFF;	// OUT_T_MSB Register @ SUB 0x04  ---  TD[11:4]		// The integer part of the temperature value.
		alt_data[4] = getI2C(1) & 0xF0; // OUT_T_LSB Register @ SUB 0x05  ---  TD[3:0]			// The upper nybble is the fractional part of the temperature value.
    stopI2C();
		
//		int sign = alt_data[0] > 0x7F;
		
    print_alt_data(alt_data);
    return alt_data;
}

typedef struct {
	float altitude_m;    // Altitude in m
	float temperature_C; // Temperature in C
	float temperature_F; // Temperature in F
} alt_data_struct;

alt_data_struct convert_alt_data(sym * data) {
	alt_data_struct alt = {};
	alt.altitude_m = (float)((int)data[0]<<8 | (int)data[1]) + (float)(data[2]/16)/16;
	alt.temperature_C = (float)data[3] + (float)(data[4]/16)/16.0;
	alt.temperature_F = alt.temperature_C * 9.0/5.0 + 32.0;
	return alt;
}

void print_alt_data(sym * data) {
    __XC_UART = __PC_UART;
		alt_data_struct alt = convert_alt_data(data);
		if (verbose_mode) { printf("\t\t\t\t\t\t\t\t"); } printf("\t" "alt = %3.2f meters above sea level\n\r", alt.altitude_m);
		if (verbose_mode) { printf("\t\t\t\t\t\t\t\t"); } printf("\t" "T   = %3.2f*F (%3.2f*C)\n\r", alt.temperature_F, alt.temperature_C);
}

#endif	/* ALT_CONFIG_H */

