/* 
 * File:   ble_config.h
 * Author: cstgeo
 *
 * Created on March 30, 2018, 8:00 PM
 */

#ifndef BLE_CONFIG_H
#define	BLE_CONFIG_H
#include "config.h"


typedef enum { BLE_STATE_CONFIGURING,
               BLE_STATE_TRANSCEIVER_ENABLED,
               BLE_STATE_CONFIGURATION_ERROR
} BLE_STATE;
BLE_STATE ble_1_state = BLE_STATE_CONFIGURING;
BLE_STATE ble_2_state = BLE_STATE_CONFIGURING;
BLE_STATE ble_3_state = BLE_STATE_CONFIGURING;

typedef enum { BLE_CONFIGURATION_STATE_NOT_CONFIGURED,
               /*CONFIG_STATES*/
               BLE_CONFIGURATION_STATE_CONFIGURATION_COMPLETE,
               /*CONFIG_ERROR_STATES*/
} BLE_CONFIGURATION_STATE;
BLE_CONFIGURATION_STATE ble_1_configuration_state = BLE_CONFIGURATION_STATE_NOT_CONFIGURED;
BLE_CONFIGURATION_STATE ble_2_configuration_state = BLE_CONFIGURATION_STATE_NOT_CONFIGURED;
BLE_CONFIGURATION_STATE ble_3_configuration_state = BLE_CONFIGURATION_STATE_NOT_CONFIGURED;

typedef enum { BLE_UART_DISCONNECTED,
							 BLE_UART_CONNECTED,
							 BLE_UART_TRANSMITTING_STRING,
			 /* -> */BLE_UART_STRING_TRANSMITTED,
			 /* -> */BLE_UART_RECEIVING_STRING,
			 /* -> */BLE_UART_STRING_RECEIVED,
			 /* -> */BLE_UART_STRING_READ
} BLE_UART_STATE;
BLE_UART_STATE ble_1_uart_state = BLE_UART_DISCONNECTED;
BLE_UART_STATE ble_2_uart_state = BLE_UART_DISCONNECTED;
BLE_UART_STATE ble_3_uart_state = BLE_UART_DISCONNECTED;

typedef enum { BLE_TRANSCEIVER_STATE_NOT_ENABLED, BLE_TRANSCEIVER_STATE_IDLE,
               BLE_TRANSCEIVER_STATE_SCANNING,/* -> */BLE_TRANSCEIVER_STATE_SCAN_COMPLETE,
               BLE_TRANSCEIVER_STATE_RECEPTION_STARTING,/* -> */BLE_TRANSCEIVER_STATE_RECEIVING
} BLE_TRANSCEIVER_STATE;
BLE_TRANSCEIVER_STATE ble_1_transceiver_state = BLE_TRANSCEIVER_STATE_NOT_ENABLED;
BLE_TRANSCEIVER_STATE ble_2_transceiver_state = BLE_TRANSCEIVER_STATE_NOT_ENABLED;
BLE_TRANSCEIVER_STATE ble_3_transceiver_state = BLE_TRANSCEIVER_STATE_NOT_ENABLED;

#define ble_initialized_condition ble_received_string[0] == 'R'
#define ble_mac_paused_condition ble_received_string[0] == '4'
#define ble_radio_power_set_condition ble_received_string[0] == 'o'
#define ble_watchdog_timer_set_condition ble_received_string[0] == 'o'
#define ble_rx_receiving_condition ble_received_string[0] == 'o'
#define ble_tx_transmitting_condition ble_received_string[0] == 'o'
#define ble_rx_received_condition ble_received_string[0] == 'r'
#define ble_tx_transmitted_condition ble_received_string[0] == 'r'





//void function_that_does_something_with_data(unsigned char data[]);

void BLE() {
  if (ble_transceiver_state == BLE_TRANSCEIVER_STATE_IDLE && ble_uart_state == BLE_UART_STRING_READ) {
   radio_rx(0);
  } else
  if (ble_uart_state == BLE_UART_STRING_RECEIVED) {
   ble_uart_state = BLE_UART_STRING_READ;
   printf("[BLE -> PIC32]:\t%s", ble_received_string);
   switch (ble_transceiver_state) {
    case BLE_TRANSCEIVER_STATE_NOT_ENABLED:
		{ 
			switch (ble_configuration_state)
			{
        case BLE_CONFIGURATION_STATE_NOT_CONFIGURED: { if (ble_initialized_condition       ) { set_baud(3,0);         } break; }
        case SETTING_WATCHDOG_TIMER: {								 if (ble_watchdog_timer_set_condition) { 
          ble_configuration_state = BLE_CONFIGURATION_STATE_CONFIGURATION_COMPLETE;
          printf("\t...BLE configuration complete...\n\r");
        } break;
				}
      }
		  break;
		}
    case BLE_TRANSCEIVER_STATE_TRANSMISSION_STARTING: if (ble_tx_transmitting_condition) { ble_transceiver_state = TRANSMITTING; } break;
    case RECEPTION_STARTING: if (ble_rx_receiving_condition) {
			      ble_transceiver_state = RECEIVING;
			      printf("\t...BLE module waiting to receive...\n\r");
			     } break;
    case TRANSMITTING: if (ble_tx_transmitted_condition) {
			ble_transceiver_state = BLE_TRANSCEIVER_STATE_IDLE;
		       } break;
    case RECEIVING: if (ble_rx_received_condition) {
		     ble_transceiver_state = BLE_TRANSCEIVER_STATE_IDLE; 
		     function_that_does_something_with_data(ble_received_string + ble_received_data_offset);
		    } break;
   }
  }
}
#endif	/* BLE_CONFIG_H */

