/* 
 * File:   config_pins.h
 * Author: cstgeo
 *
 * Created on December 29, 2017, 11:42 AM
 */

#ifndef CONFIG_PINS_H
#define	CONFIG_PINS_H
#include "config.h"

#define low 0									/*! @define low									\ The logic low value (0). */
#define high 1								/*! @define high								\ The logic high value (1). */
#define output 0							/*! @define output							\ The value assigned to an IO pin's TRIS bit to set it to output mode (0). */
#define input 1								/*! @define input								\ The value assigned to an IO pin's TRIS bit to set it to input mode (1). */
#define disabled 0						/*! @define disabled						\ The value assigned to a config register bit to disable it. (0) */
#define enabled 1							/*! @define enabled							\ The value assigned to a config register bit to enable it. (1) */
#define active_high_disable 0	/*! @define active_high_disable	\ The value assigned to an active high bit/pin to disable it (0). */
#define active_high_enable 1	/*! @define active_high_enable	\ The value assigned to an active high bit/pin to enable it (1). */
#define active_low_enable 0		/*! @define active_low_enable		\ The value assigned to an active low bit/pin to enable it (0). */
#define active_low_disable 1	/*! @define active_low_disable	\ The value assigned to an active low bit/pin to disable it (1). */


#define FMD FMD /*! @define FMD	\ The Front Motor Driver (FMD). */
#define RMD RMD /*! @define RMD	\ The Rear Motor Driver (RMD). */


// Pin Mapping
#if defined Rover
#define PORT_ALT_INT2		C
#define PORT_ALT_INT1		C
#define PORT_GAM_DRDY		B
#define PORT_GA_INT1		B
#define PORT_GA_INT2		B
#define PORT_M_INT			B
#define PORT_BLE_nRESET	B
#define PORT_VBUSON			B
#define PORT_MUX_RX			G
#define PORT_FTDI_TX		G
#define PORT_FTDI_RX		G
#define PORT_MUX_TX			G
#define PORT_GPS_n3DFIX	B
#define PORT_SRV_CTRL		D
#define PORT_FMD_ADIR		D
#define PORT_FMD_BDIR		D
#define PORT_RMD_ADIR		D
#define PORT_LORA_TX		F
#define PORT_LORA_RX		F
#elif defined BaseStation

#define PORT_PB2					B
#define PORT_PB3					B
#define PORT_PB4					B
#define PORT_PB5					B
#define PORT_PB6					B
#define PORT_PB12					B
#define PORT_VBUSON				B

#define PORT_PB0					C
#define PORT_PB1					C

#define PORT_PB13					D
#define PORT_PB14					D
#define PORT_PB15					D
#define PORT_PB16					D

#define PORT_LORA_nRESET	E

#define PORT_USBID				F
#define PORT_LORA_TX			F
#define PORT_LORA_RX			F

#define PORT_D_MINUS			G
#define PORT_D_PLUS				G
#define PORT_FTDI_TX			G
#define PORT_FTDI_RX			G
#define PORT_PB8					G
#define PORT_PB11					G

#endif

#define GET_PORT(PIN) PORT_##PIN

/* B */
typedef union {
  struct {
#if defined Rover
    unsigned GAM_DRDY:1;  // I-0
    unsigned GA_INT1:1;   // I-0
    unsigned GA_INT2:1;   // I-0
    unsigned M_INT:1;     // I-0
    unsigned BLE_nRESET:1;// O-1
    unsigned VBUSON:1;    // I-0
    unsigned SRV_ANG0:1;  // I-0-AN
    unsigned SRV_ANG1:1;  // I-0-AN
    unsigned GPS_TX:1;    // I-0
    unsigned GA_DEN:1;    // O-1
    unsigned :2;
    unsigned GPS_nRESET:1;// O-1
    unsigned GPS_1PPS:1;  // I-0
    unsigned GPS_RX:1;    // O-0
    unsigned GPS_n3DFIX:1;// I-0
#elif defined BaseStation
		unsigned PB2:1;   // I-CN
    unsigned PB3:1;   // I-CN
    unsigned PB4:1;   // I-CN
    unsigned PB5:1;   // I-CN
    unsigned PB6:1;   // I-CN
    unsigned VBUSON:1;// I-0
    unsigned :9;
    unsigned PB12:1;  // I-CN
#endif
  };
  struct {
    unsigned :32;
  };
} __B;
extern volatile __B B_IO __asm__ ("TRISB") __attribute__((section("sfrs"), address(0xBF886040)));
extern volatile __B B_R __asm__ ("PORTB") __attribute__((section("sfrs"), address(0xBF886050)));
extern volatile __B B __asm__ ("LATB") __attribute__((section("sfrs"), address(0xBF886060)));


/* C */
typedef union {
  struct {
#if defined Rover
    unsigned :12;
    unsigned C1_GATE:1;  // O-0
    unsigned ALT_INT1:1; // I-CN
    unsigned ALT_INT2:1; // I-CN
    unsigned C2_GATE:1;  // O-0
#elif defined BaseStation
		unsigned :13;
    unsigned PB1:1; // I
    unsigned PB0:1; // I
    unsigned :1;
#endif
  };
  struct {
    unsigned :32;
  };
} __C;
extern volatile __C C_IO __asm__ ("TRISC") __attribute__((section("sfrs"), address(0xBF886080)));
extern volatile __C C_R __asm__ ("PORTC") __attribute__((section("sfrs"), address(0xBF886090)));
extern volatile __C C __asm__ ("LATC") __attribute__((section("sfrs"), address(0xBF8860A0)));

/* D */
typedef union {
  struct {
#if defined Rover
		
//    unsigned FMD_ASPD:1; // O-0
//    unsigned FMD_BSPD:1; // O-0
//    unsigned RMD_ASPD:1; // O-0
//    unsigned RMD_BSPD:1; // O-0
		unsigned MD_SPD:4;  // 0b[FMD_A][FMD_B][RMD_A][RMD_B]
    unsigned SRV_CTRL:1; // O-0
//    unsigned FMD_ADIR:1; // O-0
//    unsigned FMD_BDIR:1; // O-0
//    unsigned RMD_ADIR:1; // O-0
//    unsigned RMD_BDIR:1; // O-0
		unsigned MD_DIR:4;  // 0b[FMD_A][FMD_B][RMD_A][RMD_B]
    unsigned SDA1:1;     // O-0
    unsigned SCL1:1;     // O-0
    unsigned MD_MODE:1;  // O-0
#elif defined BaseStation
		unsigned :4;
    unsigned PB13:1; // I-CN
    unsigned PB14:1; // I-CN
    unsigned PB15:1; // I-CN
    unsigned PB16:1; // I-CN
    unsigned :4;
#endif
  };
  struct {
    unsigned :32;
  };
} __D;
extern volatile __D D_IO __asm__ ("TRISD") __attribute__((section("sfrs"), address(0xBF8860C0)));
extern volatile __D D_R __asm__ ("PORTD") __attribute__((section("sfrs"), address(0xBF8860D0)));
extern volatile __D D __asm__ ("LATD") __attribute__((section("sfrs"), address(0xBF8860E0)));

/* E */
/**
 
 */
typedef union {
  struct {
#if defined Rover
    unsigned MUX_S:2;				// O-0b00
    unsigned MUX_nEN:1;     // O-0
    unsigned LORA_nRESET:1; // O-1
//    unsigned FMD_ABRKEN:1;  // O-0
//    unsigned FMD_BBRKEN:1;  // O-0
//    unsigned RMD_ABRKEN:1;  // O-0
//    unsigned RMD_BBRKEN:1;  // O-0
		unsigned MD_BRKEN:4; // 0b[FMD_A][FMD_B][RMD_A][RMD_B]
#elif defined BaseStation
		unsigned :3;
    unsigned LORA_nRESET:1; // O-1
    unsigned :4;
#endif
  };
  struct {
    unsigned :32;
  };
} __E;
extern volatile __E E_IO __asm__ ("TRISE") __attribute__((section("sfrs"), address(0xBF886100)));
extern volatile __E E_R __asm__ ("PORTE") __attribute__((section("sfrs"), address(0xBF886110)));
extern volatile __E E __asm__ ("LATE") __attribute__((section("sfrs"), address(0xBF886120)));

/* F */
 typedef union {
  struct {
#if defined Rover
    unsigned :3;
    unsigned USBID:1;   // I
    unsigned LORA_TX:1; // I
    unsigned LORA_RX:1; // O-0
#elif defined BaseStation
		unsigned :3;
    unsigned USBID:1;   // I-0
    unsigned LORA_TX:1; // I-0
    unsigned LORA_RX:1; // O-0
#endif
  };
  struct {
    unsigned :32;
  };
} __F;
extern volatile __F F_IO __asm__ ("TRISF") __attribute__((section("sfrs"), address(0xBF886140)));
extern volatile __F F_R __asm__ ("PORTF") __attribute__((section("sfrs"), address(0xBF886150)));
extern volatile __F F __asm__ ("LATF") __attribute__((section("sfrs"), address(0xBF886160)));

/* G */
typedef union {
  struct {
#if defined Rover
    unsigned :2;
    unsigned D_MINUS:1; // I
    unsigned D_PLUS:1;  // I
    unsigned :2;
    unsigned MUX_RX:1;  // I
    unsigned FTDI_TX:1; // O-0
    unsigned FTDI_RX:1; // I-0
    unsigned MUX_TX:1;  // O-0
#elif defined BaseStation
		unsigned :2;
    unsigned D_MINUS:1; // I
    unsigned D_PLUS:1;  // I
    unsigned :2;
    unsigned PB8:1;			// I-CN
    unsigned FTDI_TX:1; // O-0
    unsigned FTDI_RX:1; // I
    unsigned PB11:1;		// I-CN
#endif
  };
  struct {
    unsigned :32;
  };
} __G;
extern volatile __G G_IO __asm__ ("TRISG") __attribute__((section("sfrs"), address(0xBF886180)));
extern volatile __G G_R __asm__ ("PORTG") __attribute__((section("sfrs"), address(0xBF886190)));
extern volatile __G G __asm__ ("LATG") __attribute__((section("sfrs"), address(0xBF8861A0)));


struct {
  __B B;
	__C C;
	__D D;
	__E E;
	__F F;
	__G G;
} __PORT;


// Analog

typedef union {
  struct {
    unsigned :16;
  };
  struct {
#if defined Rover
    unsigned :6;
    unsigned SRV_ANG0:1; // AN6
    unsigned SRV_ANG1:1; // AN7
    unsigned :8;
#elif defined BaseStation
		unsigned :16;
#endif
  };
  struct {
    unsigned :32;
  };
} __ADPCFG;
extern volatile __ADPCFG ADPCFG __asm__ ("AD1PCFG") __attribute__((section("sfrs"), address(0xBF809060)));
#define ADPCFG_INIT ADPCFG_INIT/**/


#define _Input_IO(PIN) CAT(GET_PORT(PIN),_IO).PIN = 1;
#define Input_IO(...) FOR_EACH(_Input_IO,__VA_ARGS__)
#define _Output_IO_High(PIN) CAT(GET_PORT(PIN),_IO).PIN = 0;\
													GET_PORT(PIN).PIN = 1;
#define Output_IO_High(...) FOR_EACH(_Output_IO_High,__VA_ARGS__)
#define _Output_IO_Low(PIN) CAT(GET_PORT(PIN),_IO).PIN = 0;\
													GET_PORT(PIN).PIN = 0;
#define Output_IO_Low(...) FOR_EACH(_Output_IO_Low,__VA_ARGS__)

void configure_pins() {
#if defined(Rover)
    B_IO.GAM_DRDY = 1;  // I-0
    B_IO.GA_INT1 = 1;   // I-0
    B_IO.GA_INT2 = 1;   // I-0
    B_IO.M_INT = 1;     // I-0
    B_IO.BLE_nRESET = 0;// O-1
    B_IO.VBUSON = 1;    // I-0
    B_IO.SRV_ANG0 = 1;  // I-0-AN
    B_IO.SRV_ANG1 = 1;  // I-0-AN
    B_IO.GPS_TX = 1;    // I-0
    B_IO.GA_DEN = 0;    // O-1
    B_IO.GPS_nRESET = 0;// O-1
    B_IO.GPS_1PPS = 1;  // I-0
    B_IO.GPS_RX = 0;    // O-0
    B_IO.GPS_n3DFIX = 1;// I-0 
		
		C_IO.C1_GATE = 0;  // O-0
    C_IO.ALT_INT1 = 1; // I-0
    C_IO.ALT_INT2 = 1; // I-0
    C_IO.C2_GATE = 0;  // O-0  
		
//		D_IO.FMD_ASPD = 0; // O-0
//    D_IO.FMD_BSPD = 0; // O-0
//    D_IO.RMD_ASPD = 0; // O-0
//    D_IO.RMD_BSPD = 0; // O-0
		D_IO.MD_SPD = 0b0000;
    D_IO.SRV_CTRL = 0; // O-0
//    D_IO.FMD_ADIR = 0; // O-0
//    D_IO.FMD_BDIR = 0; // O-0
//    D_IO.RMD_ADIR = 0; // O-0
//    D_IO.RMD_BDIR = 0; // O-0
		D_IO.MD_DIR = 0b0000;
    D_IO.SDA1 = 0;     // O-0
    D_IO.SCL1 = 0;     // O-0
    D_IO.MD_MODE = 0;  // O-0
		
		E_IO.MUX_S = 0b00;    // O-0b00
    E_IO.MUX_nEN = 0;     // O-0
    E_IO.LORA_nRESET = 0; // O-1
//    E_IO.FMD_ABRKEN = 0;  // O-0
//    E_IO.FMD_BBRKEN = 0;  // O-0
//    E_IO.RMD_ABRKEN = 0;  // O-0
//    E_IO.RMD_BBRKEN = 0;  // O-0  
		E_IO.MD_BRKEN = 0b0000;
		
    F_IO.USBID = 1;   // I-0
    F_IO.LORA_TX = 0; // I-0
    F_IO.LORA_RX = 0; // O-0  
		
		G_IO.D_MINUS = 1; // I-0
    G_IO.D_PLUS = 1;  // I-0
    G_IO.MUX_RX = 1;  // I-0
    G_IO.FTDI_TX = 0; // O-0
    G_IO.FTDI_RX = 1; // I-0
    G_IO.MUX_TX = 0;  // O-0  
#elif defined(BaseStation)
		Input_IO(PB0,PB1,PB2,PB3,PB4,PB5,PB6,PB8,PB11,PB12,PB13,PB14,PB15,PB16,VBUSON,USBID,D_MINUS,D_PLUS)
		Output_IO_Low(LORA_nRESET)
#endif
}

#endif	/* CONFIG_PINS_H */

