/* 
 * File:   gam_config.h
 * Author: cstgeo
 *
 * Created on April 27, 2018, 1:09 PM
 */

#ifndef GAM_CONFIG_H
#  define	GAM_CONFIG_H
#include "config.h"

void configure_gam() {
    startI2C();
    putI2C(GAM_WRITE & 0xFE);
    putI2C(GA_SUB.CTRL_REG5_XL);
    putI2C(0b00111000);
    stopI2C();
    startI2C();
    putI2C(GAM_WRITE & 0xFE);
    putI2C(GA_SUB.CTRL_REG6_XL);
    putI2C(0b11000000);
		stopI2C();
}

sym gam_data[GAM_REG_COUNT+1] = {0};
sym * read_gam(void) {
    int check = 0x00;
    while ((check & 0x01) == 0) {
        startI2C(); 
				putI2C(GAM_WRITE);
				putI2C(GA_SUB.STATUS_REG2);
				startI2C();
				putI2C(GAM_READ);
        check = getI2C(1);
        stopI2C();
    }
		
    startI2C();
    putI2C(GAM_WRITE);
    putI2C(GA_SUB.OUT_X_L_XL);
		
    startI2C();
    putI2C(GAM_READ);
		gam_data[0] = getI2C(0);
		gam_data[1] = getI2C(0);
		gam_data[2] = getI2C(0);
		gam_data[3] = getI2C(0);
		gam_data[4] = getI2C(0);
		gam_data[5] = getI2C(1);
    stopI2C();
		
		print_gam_data(gam_data);
		
		return gam_data;
}

typedef struct {
	float x_g; // Acceleration in the x axis in units of Earth's gravity, g
	float y_g; // Acceleration in the y axis in units of Earth's gravity, g
	float z_g; // Acceleration in the z axis in units of Earth's gravity, g
	float x_ms2; // Acceleration in the x axis in m/s^2
	float y_ms2; // Acceleration in the y axis in m/s^2
	float z_ms2; // Acceleration in the z axis in m/s^2
} linear_acceleration;

typedef struct {
	float roll;  // Acceleration about the x axis
	float pitch; // Acceleration about the y axis
	float yaw;   // Acceleration about the z axis
} angular_acceleration;

typedef struct {
	float x; // Acceleration about the x axis
	float y; // Acceleration about the y axis
	float z; // Acceleration about the z axis
} magnetic_field;

typedef struct {
  linear_acceleration accel; // The acceleration along the x, y, and z axes
	angular_acceleration gyro; // The acceleration about the x, y, and z axes
	magnetic_field mag; // The magnetic field strength along the x, y, and z axes
} gam_data_struct;

gam_data_struct convert_gam_data(sym * data) {
	gam_data_struct gam = {{},{},{}};
	
	float g = 9.80665/*m/s^2*/;
	
	gam.accel.x_g =  ((float) twos2sign2B(data[1], data[0])*2)/((float) 0x7FFF);
  gam.accel.y_g =  ((float) twos2sign2B(data[3], data[2])*2)/((float) 0x7FFF);
  gam.accel.z_g = -((float) twos2sign2B(data[5], data[4])*2)/((float) 0x7FFF);
	gam.accel.x_ms2 = gam.accel.x_g * g;
	gam.accel.y_ms2 = gam.accel.y_g * g;
	gam.accel.z_ms2 = gam.accel.z_g * g;
	
	return gam;
}

void print_gam_data(sym * data) {
		gam_data_struct gam = convert_gam_data(data);
    __XC_UART = __PC_UART;
    if (verbose_mode) { printf("\t\t\t\t\t\t\t\t"); } printf("\t" "a_x = % 1.3f g (% 2.3f m/s^2)\n\r", gam.accel.x_g, gam.accel.x_ms2);
    if (verbose_mode) { printf("\t\t\t\t\t\t\t\t"); } printf("\t" "a_y = % 1.3f g (% 2.3f m/s^2)\n\r", gam.accel.y_g, gam.accel.y_ms2);
    if (verbose_mode) { printf("\t\t\t\t\t\t\t\t"); } printf("\t" "a_z = % 1.3f g (% 2.3f m/s^2)\n\r", gam.accel.z_g, gam.accel.z_ms2);
}

#endif	/* GAM_CONFIG_H */

