/* File:   gps_config.h
 * Author: cstgeorg
 */

#ifndef GPS_CONFIG_H
#define	GPS_CONFIG_H
#include "config.h"

typedef enum { GPS_UART_RECEIVING_STRING_TYPE,/* -> */GPS_UART_RECEIVING_STRING_TYPE_VERIFIED,/* -> */GPS_RECEIVING_STRING_DATA,/* -> */GPS_UART_STRING_RECEIVED, GPS_UART_STRING_READ
} GPS_UART_STATE;
GPS_UART_STATE gps_uart_state = GPS_UART_RECEIVING_STRING_TYPE;

typedef enum { GPS_DATA_TYPE_NONE = 0,
               GPS_DATA_TYPE_GGA  = 1,
               GPS_DATA_TYPE_GSA  = 2,  // 2
               GPS_DATA_TYPE_GSV  = 3,  // 3
               GPS_DATA_TYPE_RMC  = 4,  // 4
               GPS_DATA_TYPE_VTG  = 5,  // 5
} GPS_DATA_TYPE;
GPS_DATA_TYPE gps_data_type = GPS_DATA_TYPE_NONE;
#define print_gps_data_type printf("GPS Data Type: %d\n\n", gps_data_type)

// #define rmc_utc_time_condition c == 'R' && gps_incoming_string_index == 3

unsigned char gps_incoming_string_index;
unsigned char gps_incoming_string_current_data_section_index;
unsigned char gps_comma_count;

#define set_gps_type_verified gps_uart_state = GPS_UART_RECEIVING_STRING_TYPE_VERIFIED

#define set_gps_gga_data_type gps_data_type = GPS_DATA_TYPE_GGA; set_gps_type_verified
#define set_gps_gsa_data_type gps_data_type = GPS_DATA_TYPE_GSA; set_gps_type_verified
#define set_gps_gsv_data_type gps_data_type = GPS_DATA_TYPE_GSV; set_gps_type_verified
#define set_gps_rmc_data_type gps_data_type = GPS_DATA_TYPE_RMC; set_gps_type_verified
#define set_gps_vtg_data_type gps_data_type = GPS_DATA_TYPE_VTG; set_gps_type_verified

//index:345
//      |||
#define gga_gps_data_type_condition (c == 'G' && gps_incoming_string_index == 4)
#define gsa_gps_data_type_condition (c == 'A' && gps_incoming_string_index == 5) /* this works because it does not check index 5 if gga type*/
#define gsv_gps_data_type_condition (c == 'V' && gps_incoming_string_index == 5)
#define rmc_gps_data_type_condition (c == 'R' && gps_incoming_string_index == 3)
#define vtg_gps_data_type_condition (c == 'V' && gps_incoming_string_index == 3)

#define set_type_if_gps_data_type(DT) if DT##_gps_data_type_condition { set_gps_##DT##_data_type; }

typedef enum { GPS_INITIALIZING,
               GPS_CONFIGURING,
               GPS_TRANSCEIVER_ENABLED,
               GPS_INITIALIZATION_ERROR,
               GPS_CONFIGURATION_ERROR
} GPS_STATE;
GPS_STATE gps_state = GPS_INITIALIZING;



void do_this_after_getting_gps_data();

//#define gga_gps_data_type_condition (gps_received_string[4] == 'G')
//#define gsa_gps_data_type_condition (gps_received_string[4] == 'S' && gps_received_string[5] == 'A')
//#define gsv_gps_data_type_condition (gps_received_string[5] == 'V')
//#define rmc_gps_data_type_condition (gps_received_string[3] == 'R')
//#define vtg_gps_data_type_condition (gps_received_string[3] == 'V')

void GPS() {
 if (gps_uart_state == GPS_UART_STRING_RECEIVED)
 {
   gps_uart_state = GPS_UART_STRING_READ;
//	 do_this_after_getting_gps_data();
 }
}

typedef struct {
	sym h; // hours
	sym m; // minutes
	float s; // seconds
} time;

typedef struct {
	sym d; // degrees
	sym m; // minutes
	float s; // seconds
} angle;

//typedef struct {
//	
//};

typedef struct {
	time utc_time;   // Coordinated Universal Time (UTC)
	angle latitude;  // The latitude on earth
	sym ns_indicator;// North or South
	angle longitude; // The longitude on earth
	sym ew_indicator;// East or West
	float s_kn;			 // The speed over the ground in knots
	float s_ms;			 // The speed over the ground in meters per second
//	sym string[80];
} gps_data_struct;

gps_data_struct convert_gps_data(sym * data) {
	gps_data_struct gps = {{},{},' ',{},' ',0,0};
	gps.utc_time.h	 = (data[0]-'0')*10 + (data[1]-'0'); // [hh][mm][ss].[sss]
	gps.utc_time.m	 = (data[2]-'0')*10 + (data[3]-'0');
	gps.utc_time.s	 = (float)(data[4]-'0')*10 + (data[5]-'0') + (float)((data[6]-'0')*100 + (data[7]-'0')*10 + (data[8]-'0'))*0.001;
	
	gps.latitude.d	 = data[4]-'0';
	gps.latitude.m	 =  data[5]-'0';
	gps.latitude.s	 = (float)(data[6]-'0')*0.006; // [dd][mm].[mmmm]
	
	gps.ns_indicator = data[7];
	
	gps.longitude.d	 = data[8]-'0'; 
	gps.longitude.d	 = data[9]-'0'; 
	gps.longitude.d	 = (float)(data[10]-'0')*0.006; // [ddd][mm].[mmmm]
	
	gps.ew_indicator = data[11];
	
	gps.s_kn = (float)data[12]/1000;
	gps.s_ms = gps.s_kn * 1852.0 / 3600.0;
//	gps.altitude_m = (float)((int)data[0]<<8 | (int)data[1]) + (float)(data[2]/16)/16;
	
	return gps;
}

void print_gps_data(sym * data) {
    __XC_UART = __PC_UART;
		gps_data_struct gps = convert_gps_data(data);
		if (verbose_mode) { printf("\t\t\t\t\t\t\t\t"); } printf("\t" "UTC Time  = %d:%d:%02.3f\n\r", gps.utc_time.h, gps.utc_time.m, gps.utc_time.s);
		if (verbose_mode) { printf("\t\t\t\t\t\t\t\t"); } printf("\t" "Latitude  = %d:%d:%02.3f\n\r", gps.latitude.d, gps.latitude.m, gps.latitude.s);
		if (verbose_mode) { printf("\t\t\t\t\t\t\t\t"); } printf("\t" "Longitude = %d:%d:%02.3f\n\r", gps.longitude.d, gps.longitude.m, gps.longitude.s);
}

//case GPS_RECEIVING_STRING_DATA: {
//						if (gps_data_type == GPS_DATA_TYPE_RMC) {
//							if (gps_comma_count ==  1) { RMC_data.UTC_Time[gps_incoming_string_current_data_section_index++] = c;
//																					 RMC_data.UTC_Time[gps_incoming_string_current_data_section_index  ] = 0; } else
//							if (gps_comma_count ==  2) { RMC_data.Status = c; } else
//							if (gps_comma_count ==  3) { RMC_data.Latitude[gps_incoming_string_current_data_section_index++] = c;
//																					 RMC_data.Latitude[gps_incoming_string_current_data_section_index  ] = 0; } else
//							if (gps_comma_count ==  4) { RMC_data.NS_Indicator = c; } else
//							if (gps_comma_count ==  5) { RMC_data.Longitude[gps_incoming_string_current_data_section_index++] = c;
//																					 RMC_data.Longitude[gps_incoming_string_current_data_section_index  ] = 0; } else
//							if (gps_comma_count ==  6) { RMC_data.EW_Indicator = c; } else
//							if (gps_comma_count ==  7) { RMC_data.Speed_over_Ground[gps_incoming_string_current_data_section_index++] = c; } else
//							if (gps_comma_count ==  8) { RMC_data.Course_over_Ground = c; } else
//							if (gps_comma_count ==  9) { RMC_data.Longitude[gps_incoming_string_current_data_section_index++] = c; } else
//							if (gps_comma_count == 10) { RMC_data.EW_Indicator = c; }
////							GPS_DATA_TYPE_GGA
////							GPS_DATA_TYPE_GSA
////							GPS_DATA_TYPE_GSV
////							GPS_DATA_TYPE_VTG
//						}
//            break;
//        }

#endif	/* GPS_CONFIG_H */

