/* 
 * File:   interrupt_config.h
 * Author: Chris St. George
 */

#ifndef INTERRUPT_CONFIG_H
#define	INTERRUPT_CONFIG_H
#include "config.h"

// Interrupt Flag, Priority, and Enable bits
#define Global_Interrupts_Enabled asm("ei")
#define Global_Interrupts_Disabled asm("di")
#define Multivector_Interrupts_Enable INTCONbits.MVEC

// Interrupt Function Macros
#define PRAGMA(x) _Pragma(#x)
#define VECTOR(x) _##x##_VECTOR
#define _PRIORITY(x) ipl ## x ## auto
#define PRIORITY(x) _PRIORITY(x)

#define __Configure_Interrupts(X) configure_##X##_interrupts()
#define _Configure_Interrupts(X) __Configure_Interrupts(X);
#define Configure_Interrupts(...)\
 void configure_interrupts() {\
	printConfigurationStarted(Interrupts);\
  Global_Interrupts_Enabled;								printf("\n\r\t" "Global Interrupts Enabled");\
  Multivector_Interrupts_Enable = enabled;  printf("\n\r\t" "Multivector Interrupts Enabled");\
  FOR_EACH(_Configure_Interrupts,__VA_ARGS__)\
	printConfigurationCompleted(Interrupts);\
 }

#define _Interrupt(object, full_name, priority, subpriority)\
	void __Configure_Interrupts(object) {\
  printf("\n\r\t" "configure_" #object "_interrupts()");\
	printf("\n\r\t\t" #object "_Interrupt_Priority = " #priority);\
	printf("\n\r\t\t" #object "_Interrupt_Subpriority = " #subpriority);\
  object##_Interrupt_Priority = priority; /* Sets the priority for the interrupt */\
  object##_Interrupt_Subpriority = subpriority; /* Sets the subpriority for the interrupt */
#define Interrupt(object, full_name, priority, subpriority, event_type, is_enabled)\
	_Interrupt(object, full_name, priority, subpriority)\
	printf("\n\r\t\t" #object "_Interrupt_Enable = " #is_enabled);\
  object##_Interrupt_Flag = false; /* Sets the initial flag state for the interrupt */\
  object##_Interrupt_Enable = is_enabled; /* Sets the enable bit for the interrupt */\
 }\
 __attribute__((vector(VECTOR(full_name)),interrupt(PRIORITY(priority)), nomips16))\
 void object##_##event_type##_Event()


//__interrupt()

/*<><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><>*/
#include "interrupt_config_Timer.h"
#include "interrupt_config_UART.h"

#if defined(Rover)
#include "interrupt_config_IIC.h"
#endif

#include "interrupt_config_Change_Notification.h"

Configure_Interrupts(
		Timer,
		UART,
#if defined(Rover)
		IIC,
#endif
		Change_Notification
)

#endif	/* INTERRUPT_CONFIG_H */

