/* 
 * File:   interrupt_config_change_notification.h
 * Author: cstgeo
 *
 * Created on February 11, 2018, 2:43 PM
 */

#ifndef INTERRUPT_CONFIG_CHANGE_NOTIFICATION_H
#define	INTERRUPT_CONFIG_CHANGE_NOTIFICATION_H
#include "config.h"

#define Change_Notification_Vector _CHANGE_NOTICE_VECTOR
#define Change_Notification_Interrupt_Flag IFS1bits.CNIF
#define Change_Notification_Interrupt_Priority IPC6bits.CNIP
#define Change_Notification_Interrupt_Subpriority IPC6bits.CNIS
#define Change_Notification_Interrupt_Enable IEC1bits.CNIE
#define Change_Notification_On CNCONbits.ON

typedef union {
  struct {
#if defined Rover
    unsigned ALT_INT2:1;	// CNEN0	/ RC14
    unsigned ALT_INT1:1;	// CNEN1	/ RC13
    unsigned GAM_DRDY:1;	// CNEN2	/ RB0
    unsigned GA_INT1:1;		// CNEN3	/ RB1
    unsigned GA_INT2:1;		// CNEN4	/ RB2
    unsigned M_INT:1;			// CNEN5	/ RB3
    unsigned BLE_nRESET:1;// CNEN6	/ RB4
    unsigned VBUSON:1;		// CNEN7	/ RB7
    unsigned MUX_RX:1;		// CNEN8	/ RG6
    unsigned FTDI_TX:1;		// CNEN9	/ RG7
    unsigned FTDI_RX:1;		// CNEN10	/ RG8
    unsigned MUX_TX:1;		// CNEN11 / RG9
    unsigned GPS_n3DFIX:1;// CNEN12 / RB15
    unsigned SRV_CTRL:1;	// CNEN13 / RD4
    unsigned FMD_ADIR:1;	// CNEN14 / RD5
    unsigned FMD_BDIR:1;	// CNEN15 / RD6
    unsigned RMD_ADIR:1;	// CNEN16 / RD7
    unsigned LORA_TX:1;		// CNEN17 / RF4
    unsigned LORA_RX:1;		// CNEN18 / RF5
#elif defined BaseStation
		unsigned PB0:1; // CNEN0 / RC14
    unsigned PB1:1; // CNEN1 / RC13
    unsigned PB2:1; // CNEN2 / RB0
    unsigned PB3:1; // CNEN3 / RB1
    unsigned PB4:1; // CNEN4 / RB2
    unsigned PB5:1; // CNEN5 / RB3
    unsigned PB6:1; // CNEN6 / RB4
    unsigned VBUSON:1; // CNEN7 / RB7
    unsigned PB8:1; // CNEN8 / RG6
    unsigned FTDI_TX:1; // CNEN9 / RG7
    unsigned FTDI_RX:1; // CNEN10 / RG8
    unsigned PB11:1; // CNEN11 / RG9
    unsigned PB12:1; // CNEN12 / RB15
    unsigned PB13:1; // CNEN13 / RD4
    unsigned PB14:1; // CNEN14 / RD5
    unsigned PB15:1; // CNEN15 / RD6
    unsigned PB16:1; // CNEN16 / RD7
    unsigned LORA_TX:1; // CNEN17 / RF4
    unsigned LORA_RX:1; // CNEN18 / RF5
#endif
  };
  struct {
    unsigned :32;
  };
} __CN;
extern volatile __CN CN __asm__ ("CNEN") __attribute__((section("sfrs"), address(0xBF8861D0)));

#define _Paste(X) X;
#define _CN_Enable(X) CNENbits.CNEN##X = 1;
#define CN_Enable(...) FOR_EACH(_CN_Enable,__VA_ARGS__)
#define _CNPU_Enable(X) CNPUEbits.CNPUE##X = 1;
#define CNPU_Enable(...) FOR_EACH(_CNPU_Enable,__VA_ARGS__)
#define _CN_Disable(X) CNENbits.CNEN##X = 0;
#define CN_Disable(...) FOR_EACH(_CN_Disable,__VA_ARGS__)
#define _CNPU_Disable(X) CNPUEbits.CNPUE##X = 0;
#define CNPU_Disable(...) FOR_EACH(_CNPU_Disable,__VA_ARGS__)
#define _Enable(X) X = enabled;
#define Enable(...) FOR_EACH(_Enable,__VA_ARGS__)
#define _Disable(X) X = disabled;
#define Disable(...) FOR_EACH(_Disable,__VA_ARGS__)


#define _CN_Interrupt(object, full_name, priority, subpriority, event_type, is_enabled, is_on,...)\
	printf("\n\r\t\t" #object "_Interrupt_On = " #is_on);\
  /*interrupt object##_##event_type##_Event autoIPL(priority) vector VECTOR(object);*/\
  object##_Interrupt_Priority = priority; /* Sets the priority for the interrupt */\
  object##_Interrupt_Subpriority = subpriority; /* Sets the subpriority for the interrupt */\
  object##_Interrupt_Flag = false; /* Sets the initial flag state for the interrupt */\
  object##_Interrupt_Enable = is_enabled; /* Sets the enable bit for the interrupt */\
  object##_On = is_on; /* Sets the control bit on */\
  FOR_EACH(_Paste,__VA_ARGS__)\
 }\
 __attribute__((vector(VECTOR(full_name)),interrupt(PRIORITY(priority)), nomips16))\
 void object##_##event_type##_Event()
#define CN_Interrupt(priority, subpriority, event_type, is_enabled, is_on,...)\
 _Interrupt(Change_Notification, CHANGE_NOTICE, priority, subpriority)\
 _CN_Interrupt(Change_Notification, CHANGE_NOTICE, priority, subpriority, event_type, is_enabled, is_on,__VA_ARGS__)


#define Enable_CN(PIN)\
	CAT(GET_PORT(PIN),_IO).PIN = input;\
	CN.PIN = enabled
#define Disable_CN(PIN,IO)\
	CN.PIN = disabled;\
	CAT(GET_PORT(PIN),_IO).PIN = IO

typedef enum {
	None,
			
	Reset_Rover,
			
	Send_All_Data,  
	Send_LDR_Data, 
	Send_GPS_Data, 
	Send_BLE_Data, 
	Send_ALT_Data, 
	Send_GAM_Data, 
	Send_PIC_Data, 
			
	Drive_Forward,	
  Drive_Backward,
			
	Turn_Left,
	Turn_Right,
			
	Extend_PV,
	Retract_PV,
			
	Lock_PV,
	Unlock_PV,
			
	Stop_Rover,
			
	Light_Charge_1,
			
	Light_Charge_2,
			
	Set_To_Flight_Mode,
			
	Autonomous_Movement,
			
} command;

//void print_com_data(sym * data); 
command convert_com_data(sym * data) {
	return (command)data[0];
}

void print_command(command c) {
	switch (c) {
			case Reset_Rover:		            printf("< Reset Rover >"); break;
			
		  case Send_All_Data:							printf("< Send All Data >"); break;
		  case Send_LDR_Data:             printf("< Send LIDAR Data >"); break;
			case Send_GPS_Data:             printf("< Send GPS Data >"); break;
			case Send_BLE_Data:             printf("< Send Bluetooth Data >"); break;
			case Send_ALT_Data:             printf("< Send Altimeter Data >"); break;
			case Send_GAM_Data:             printf("< Send Gyroscope/Accelerometer/Magnetometer Data >"); break;
			case Send_PIC_Data:             printf("< Send PIC32 Data >"); break;
			 
			case Drive_Forward:	            printf("< Drive Forward >"); break;
		  case Drive_Backward:            printf("< Drive Backward >"); break;
			
			case Turn_Left:			            printf("< Turn Left >"); break;
			case Turn_Right:		            printf("< Turn Right >"); break;
			
			case Extend_PV:			            printf("< Extend PV >"); break;
			case Retract_PV:		            printf("< Retract PV >"); break;
			
			case Lock_PV:				            printf("< Lock PV >"); break;
			case Unlock_PV:				          printf("< Unlock PV >"); break;
			
			case Stop_Rover:		            printf("< Stop Rover >"); break;
			
		  case Light_Charge_1:            printf("< Light E-Match 1 >"); break;
			
			case Light_Charge_2:            printf("< Light E-Match 2 >"); break;
			
			case Set_To_Flight_Mode:		    printf("< Set To Flight Mode >"); break;
			
			case Autonomous_Movement:	      printf("< Begin/Pause/Resume Autonomous Movement >"); break;
			
		  default:												printf("< Unknown >"); break;
	}
}	

command last_command = None;
command data_out = Send_All_Data;

command getNextOptionForLastCommand() {
	switch (last_command) {
			case Reset_Rover:								return Reset_Rover;           
			
		  case Send_All_Data:							return Send_LDR_Data;
		  case Send_LDR_Data:             return Send_GPS_Data;
			case Send_GPS_Data:             return Send_BLE_Data;
			case Send_BLE_Data:             return Send_ALT_Data;
			case Send_ALT_Data:             return Send_GAM_Data;
			case Send_GAM_Data:             return Send_PIC_Data;
			case Send_PIC_Data:             return Send_All_Data;
			
			case Drive_Forward:	            return Drive_Backward;
		  case Drive_Backward:            return Drive_Forward;
			
			case Turn_Left:			            return Turn_Right;
			case Turn_Right:		            return Turn_Left;
			
			case Extend_PV:			            return Retract_PV;
			case Retract_PV:		            return Extend_PV;
			
			case Lock_PV:				            return Unlock_PV;
			case Unlock_PV:				          return Lock_PV;
			
			case Stop_Rover:								return Stop_Rover; 
			
		  case Light_Charge_1:            return Light_Charge_1;
			
			case Light_Charge_2:            return Light_Charge_2;
			
			case Set_To_Flight_Mode:		    return Set_To_Flight_Mode;
			
			case Autonomous_Movement:				return Autonomous_Movement;
			
			default:												return None;
	}
}

#define CommandCommitButton(PIN)\
	(CN.PIN && CAT(GET_PORT(PIN),_IO).PIN && CAT(GET_PORT(PIN),_R).PIN)\
	{\
	 if (last_command != None) {\
		command_queue.add_symbol(for_COM, (sym)last_command, true); \
		last_command = None;\
		lockCN();\
	 }\
	 if (command_queue.is_empty()) {\
   } else if (command_queue.last_chunk_pointer_index == command_queue.first_chunk_pointer_index) {\
	  set_UART_PC; printf("\r\t\t\t\t\t\t\t\t\t\t\t\t\t" "The selected command has been queued for transmission:" "\n\r");\
   } else {\
		set_UART_PC; printf("\r\t\t\t\t\t\t\t\t\t\t\t\t\t" "The selected commands have been queued for transmission:" "\n\r"); \
	 }\
	 int i;\
	 for (i=command_queue.first_chunk_pointer_index; i<=command_queue.last_chunk_pointer_index; i++) {\
		printf("\r\t\t\t\t\t\t\t\t\t\t\t\t\t\t" ); print_command((command)command_queue.chunk_pointer[i][0]); printf( "\n\r");\
	 }\
   CAT(GET_PORT(PIN),_IO).PIN = output;\
   GET_PORT(PIN).PIN = high;\
   CN.PIN = disabled;\
	 outgoing_queue.set_empty();\
   outgoing_queue.enqueue_lora_nybbles(command_queue.nybble);\
	 command_queue.set_empty();\
  }

#define CommandClearButton(PIN)\
	(CN.PIN && CAT(GET_PORT(PIN),_IO).PIN && CAT(GET_PORT(PIN),_R).PIN)\
  {\
   printf("\r\t\t\t\t\t\t\t\t\t\t\t\t\t" "Selected commands have been reset." "\n\r");\
   CAT(GET_PORT(PIN),_IO).PIN = output;\
   GET_PORT(PIN).PIN = high;\
   CN.PIN = disabled;\
	 last_command = None;\
   command_queue.set_empty();\
	 updateCN();\
  }

#define NextOptionButton(PIN)\
	(CN.PIN && CAT(GET_PORT(PIN),_IO).PIN && CAT(GET_PORT(PIN),_R).PIN)\
	{\
	 saved_cn = CNEN;\
   CN.PIN = disabled;\
	 set_UART_PC;\
	 if (last_command == None) {\
	  verbose_mode = !verbose_mode;\
	  if (verbose_mode) {\
		 printf("\r\t\t\t\t\t\t\t\t\t\t\t\t\tVerbose mode enabled.\r\n\n");\
	  } else {\
		 printf("\r\t\t\t\t\t\t\t\t\t\t\t\t\tVerbose mode disabled.\r\n\n");\
		}\
	 } else {\
	  printf("\r\t\t\t\t\t\t\t\t\t\t\t\t\tLast entered command ");\
	  print_command(last_command);\
	  if (last_command == getNextOptionForLastCommand()) {\
	   printf(" unchanged");\
	  } else {\
	   printf(" has now been changed to ");\
	   last_command = getNextOptionForLastCommand();\
	   print_command(last_command);\
	  } printf("\r\n");\
	 }\
   CN_Timer_On = true;\
  }

#define CommandButton(PIN,COMMAND)\
	(CN.PIN && CAT(GET_PORT(PIN),_IO).PIN && CAT(GET_PORT(PIN),_R).PIN)\
	{\
	 set_UART_PC; printf("\r\t\t\t\t\t\t\t\t\t\t\t\t\t"); print_command(COMMAND); printf(" Command Selected\r\n");\
	 if (last_command != None) {\
		command_queue.add_symbol(for_COM, (sym)last_command, true);\
   }\
	 last_command = COMMAND;\
   CAT(GET_PORT(PIN),_IO).PIN = output;\
   GET_PORT(PIN).PIN = high;\
   CN.PIN = disabled;\
  }

#define LatchButton(PIN,...)\
  (CN.PIN && CAT(GET_PORT(PIN),_IO).PIN && CAT(GET_PORT(PIN),_R).PIN)\
  {\
   printf("\t" #PIN "\n\r");\
   CAT(GET_PORT(PIN),_IO).PIN = output;\
   GET_PORT(PIN).PIN = high;\
   CN.PIN = disabled;\
   FOR_EACH(_Paste,__VA_ARGS__)\
  }

#define updateLatchButton(PIN)\
  if (!CN.PIN)\
  {\
   CAT(GET_PORT(PIN),_IO).PIN = input;\
	 while(CAT(GET_PORT(PIN),_R).PIN);\
   CN.PIN = enabled;\
  }

#if defined(Rover)
void queueData(char * data);
#elif defined(BaseStation)
void queueCommand(char command);
void updateCN() {
  updateLatchButton(PB0)
  updateLatchButton(PB1)
  updateLatchButton(PB2)
  updateLatchButton(PB3)
  updateLatchButton(PB4)
  updateLatchButton(PB5)
  updateLatchButton(PB6)
  updateLatchButton(PB8)
  updateLatchButton(PB11)
  updateLatchButton(PB12)
  updateLatchButton(PB13)
  updateLatchButton(PB14)
  updateLatchButton(PB15)
  updateLatchButton(PB16)
}
void lockCN() {
  CN.PB0  = false;
  CN.PB1  = false;
  CN.PB2  = false;
  CN.PB3  = false;
  CN.PB4  = false;
  CN.PB5  = false;
  CN.PB6  = false;
  CN.PB8  = false;
  CN.PB11 = false;
  CN.PB12 = false;
  CN.PB13 = false;
  CN.PB14 = false;
  CN.PB15 = false;
  CN.PB16 = false;
}
#endif
// Change Notification

#if defined(BaseStation)
bool should_transmit_commands = false;
#endif

CN_Interrupt(6,1,Interrupt,enabled,true,
#if defined(Rover)
    CN.ALT_INT2 = disabled, // CNEN0
    CN.ALT_INT1 = disabled, // CNEN1
    CN.GAM_DRDY = disabled, // CNEN2
    CN.GA_INT1 = disabled, // CNEN3
    CN.GA_INT2 = disabled, // CNEN4
    CN.M_INT = disabled, // CNEN5
    CN.BLE_nRESET = disabled, // CNEN6
    CN.VBUSON = disabled, // CNEN7
    CN.MUX_RX = disabled, // CNEN8
    CN.FTDI_TX = disabled, // CNEN9
    CN.FTDI_RX = disabled, // CNEN10
    CN.MUX_TX = disabled, // CNEN11
    CN.GPS_n3DFIX = disabled, // CNEN12
    CN.SRV_CTRL = disabled, // CNEN13
    CN.FMD_ADIR = disabled, // CNEN14
    CN.FMD_BDIR = disabled, // CNEN15
    CN.RMD_ADIR = disabled, // CNEN16
    CN.LORA_TX = disabled, // CNEN17
    CN.LORA_RX = disabled // CNEN18
#elif defined(BaseStation)
    Enable_CN(PB0), // CNEN0
    Enable_CN(PB1), // CNEN1
    Enable_CN(PB2), // CNEN2
    Enable_CN(PB3), // CNEN3
    Enable_CN(PB4), // CNEN4
    Enable_CN(PB5), // CNEN5
    Enable_CN(PB6), // CNEN6
    Enable_CN(PB8), // CNEN8
    Enable_CN(PB11),// CNEN11
    Enable_CN(PB12),// CNEN12
    Enable_CN(PB13),// CNEN13
    Enable_CN(PB14),// CNEN14
    Enable_CN(PB15),// CNEN15
    Enable_CN(PB16),// CNEN16
#endif
)
{
 // Check interrupt pins here.
// printf("\n\r" "void Change_Notification_Interrupt_Event()" "\n\r");
#if defined(Rover)
 if LatchButton(ALT_INT2)		else
 if LatchButton(ALT_INT1)		else
 if LatchButton(GAM_DRDY)		else
 if LatchButton(GA_INT1)		else
 if LatchButton(GA_INT2)		else
 if LatchButton(M_INT)      else
 if LatchButton(BLE_nRESET) else
 if LatchButton(VBUSON)			else
 if LatchButton(MUX_RX)			else
 if LatchButton(FTDI_TX)		else
 if LatchButton(FTDI_RX)		else
 if LatchButton(MUX_TX)			else
 if LatchButton(GPS_n3DFIX) else
 if LatchButton(SRV_CTRL)		else
 if LatchButton(LORA_TX)		else
 if LatchButton(LORA_RX)		else
 { printf("\t[!] - An unexpected CNI event occurred" "\n\r"); }
#elif defined(BaseStation)
 if CommandButton(PB0, Reset_Rover) else
 if CommandButton(PB1, Send_All_Data) else
 if CommandButton(PB2, Drive_Forward) else
 if CommandButton(PB3, Turn_Left) else
 if CommandButton(PB4, Extend_PV) else
 if CommandButton(PB5, Lock_PV) else
 if CommandButton(PB8, Stop_Rover) else
 if CommandButton(PB11,Light_Charge_1) else
 if CommandButton(PB12,Light_Charge_2) else
 if CommandButton(PB13,Set_To_Flight_Mode) else
 if CommandButton(PB14,Autonomous_Movement) else
 if CommandCommitButton(PB6) else
 if NextOptionButton(PB15) else
 if CommandClearButton(PB16)
#endif
 // Clear Flag after pin check.
 Change_Notification_Interrupt_Flag = false;
}

command active_command = None;
command current_data_out_mode = Send_All_Data;

void setToFlightMode() {
	
}

void autonomousMovement() {
	T4CONbits.ON = 1;
}
void setup_devices(); 
#if defined(Rover)
void Command() {
	if (!command_queue.is_empty() && lora_transceiver_state == LORA_TRANSCEIVER_IDLE) {
		active_command = convert_com_data(command_queue.dequeue());
		switch (active_command) {
			case Reset_Rover:		           { printf("\r\t\t\t\t\t\t\t\t\t\t\t\t\t" "Executing command "); print_command(active_command); printf(" [Inactive]\n\r"); /*setup_devices();*/ break; }
		  case Send_All_Data:						 
		  case Send_LDR_Data:            
			case Send_GPS_Data:            
			case Send_BLE_Data:            
			case Send_ALT_Data:            
			case Send_GAM_Data:            
			case Send_PIC_Data:            { printf("\r\t\t\t\t\t\t\t\t\t\t\t\t\t" "Executing command "); print_command(active_command); printf("\n\r"); current_data_out_mode = active_command; active_command = None; break; }
			case Drive_Forward:	           { printf("\r\t\t\t\t\t\t\t\t\t\t\t\t\t" "Executing command "); print_command(active_command); printf("\n\r"); driveForward(); break; }
		  case Drive_Backward:           { printf("\r\t\t\t\t\t\t\t\t\t\t\t\t\t" "Executing command "); print_command(active_command); printf("\n\r"); driveBackward(); break; }
			case Turn_Left:			           { printf("\r\t\t\t\t\t\t\t\t\t\t\t\t\t" "Executing command "); print_command(active_command); printf("\n\r"); turnLeft(); break; }
			case Turn_Right:		           { printf("\r\t\t\t\t\t\t\t\t\t\t\t\t\t" "Executing command "); print_command(active_command); printf("\n\r"); turnRight(); break; }
			case Extend_PV:			           { printf("\r\t\t\t\t\t\t\t\t\t\t\t\t\t" "Executing command "); print_command(active_command); printf("\n\r"); extendPV(); break; }
			case Retract_PV:		           { printf("\r\t\t\t\t\t\t\t\t\t\t\t\t\t" "Executing command "); print_command(active_command); printf("\n\r"); retractPV(); break; }
			case Lock_PV:				           { printf("\r\t\t\t\t\t\t\t\t\t\t\t\t\t" "Executing command "); print_command(active_command); printf("\n\r"); lockPV(); break; }
			case Unlock_PV:				         { printf("\r\t\t\t\t\t\t\t\t\t\t\t\t\t" "Executing command "); print_command(active_command); printf("\n\r"); unlockPV(); break; }
			case Stop_Rover:		           { printf("\r\t\t\t\t\t\t\t\t\t\t\t\t\t" "Executing command "); print_command(active_command); printf("\n\r"); stopRover(); break; }
		  case Light_Charge_1:           { printf("\r\t\t\t\t\t\t\t\t\t\t\t\t\t" "Executing command "); print_command(active_command); printf("\n\r"); C.C1_GATE = 1; break; }
			case Light_Charge_2:           { printf("\r\t\t\t\t\t\t\t\t\t\t\t\t\t" "Executing command "); print_command(active_command); printf("\n\r"); C.C2_GATE = 1; break; }
			case Set_To_Flight_Mode:		   { printf("\r\t\t\t\t\t\t\t\t\t\t\t\t\t" "Executing command "); print_command(active_command); printf("\n\r"); setToFlightMode(); break; }
			case Autonomous_Movement:	     { printf("\r\t\t\t\t\t\t\t\t\t\t\t\t\t" "Executing command "); print_command(active_command); printf("\n\r"); autonomousMovement(); break; }
		  default:											 { printf("\r\t\t\t\t\t\t\t\t\t\t\t\t\t" "Executing command "); print_command(active_command); printf("\n\r"); break; }
		}
	} else if (active_command != None) {
		active_command = None;
	}
}
#endif

#endif	/* INTERRUPT_CONFIG_CHANGE_NOTIFICATION_H */

