/* 
 * File:   interrupt_config_iic.h
 * Author: cstgeo
 *
 * Created on March 31, 2018, 5:25 PM
 */

#ifndef INTERRUPT_CONFIG_IIC_H
#define	INTERRUPT_CONFIG_IIC_H
#include "config.h"

// Interrupt Flag, Priority, and Enable bits
#define IIC_Vector _I2C_1_VECTOR
#define IIC_Interrupt_Priority IPC6bits.I2C1IP
#define IIC_Interrupt_Subpriority IPC6bits.I2C1IS
#define IIC_Bus_Collision_Interrupt_Flag IFS0bits.I2C1BIF
#define IIC_Slave_Interrupt_Flag IFS0bits.I2C1SIF
#define IIC_Master_Interrupt_Flag IFS0bits.I2C1AMIF
#define IIC_Bus_Collision_Interrupt_Enable IEC0bits.I2C1ABIE
#define IIC_Master_Interrupt_Enable IEC0bits.I2C1AMIE
#define IIC_Slave_Interrupt_Enable IEC0bits.I2C1ASIE
#define IIC_Acknowledge_Status I2C1STATbits.ACKSTAT
//#define IIC_ I2C1STATbits.

#define _Configure_IIC_Interrupts(X)  _Configure_Interrupts(X##_IIC)
#define Configure_IIC_Interrupts(...)\
 void __Configure_Interrupts(IIC) {\
	printConfigurationStarted(IIC_Interrupts);\
  FOR_EACH(_Configure_IIC_Interrupts,__VA_ARGS__)\
	printConfigurationCompleted(IIC_Interrupts);\
 }

#define _IIC_Interrupt(object,full_name, priority, subpriority, event_type, m_is_enabled, s_is_enabled, bc_is_enabled)\
	printf("\n\r\t\t" #object "_Master_Interrupt_Enable = " #m_is_enabled);\
	printf("\n\r\t\t" #object "_Slave_Interrupt_Enable = " #s_is_enabled);\
	printf("\n\r\t\t" #object "_Bus_Collision_Interrupt_Enable = " #bc_is_enabled);\
  IIC_Master_Interrupt_Enable = m_is_enabled;\
	IIC_Slave_Interrupt_Enable = s_is_enabled;\
	IIC_Bus_Collision_Interrupt_Enable = bc_is_enabled;\
 }\
 __attribute__((vector(VECTOR(full_name)),interrupt(PRIORITY(priority)), nomips16))\
 void object##_##event_type##_Event()
#define IIC_Interrupt(bus_number, priority, subpriority, event_type, bc_is_enabled, m_is_enabled, s_is_enabled)\
 _Interrupt(IIC, I2C_##bus_number, priority, subpriority)\
 _IIC_Interrupt(IIC, I2C_##bus_number, priority, subpriority, event_type, bc_is_enabled, m_is_enabled, s_is_enabled)


IIC_Interrupt(1,6,1,Overflow,
							/*master*/disabled,
							/*slave*/disabled,
							/*bus collision*/disabled
) {
	if (IIC_Master_Interrupt_Flag && IIC_Master_Interrupt_Enable) {
		printf("\t" "IIC_Master_Interrupt Event" "\n\r");
		IIC_Master_Interrupt_Enable = false;
  } else
  if (IIC_Slave_Interrupt_Flag && IIC_Slave_Interrupt_Enable) {
		printf("\t" "IIC_Slave_Interrupt Event" "\n\r");
		IIC_Slave_Interrupt_Enable = false;
  } else
	if (IIC_Bus_Collision_Interrupt_Flag && IIC_Bus_Collision_Interrupt_Enable) {
	  printf("\t" "IIC_Bus_Collision_Interrupt Event" "\n\r");
		IIC_Bus_Collision_Interrupt_Enable = false;
  }
}


#endif	/* INTERRUPT_CONFIG_IIC_H */

