/* 
 * File:   interrupt_config_Timers.h
 * Author: cstgeo
 *
 * Created on February 11, 2018, 2:48 PM
 */

#ifndef INTERRUPT_CONFIG_TIMERS_H
#define	INTERRUPT_CONFIG_TIMERS_H
#include "config.h"

// Interrupt Flag, Priority, and Enable bits
#define Timer_1_Vector _TIMER_1_VECTOR
#define Timer_1_Interrupt_Flag IFS0bits.T1IF
#define Timer_1_Interrupt_Priority IPC1bits.T1IP
#define Timer_1_Interrupt_Subpriority IPC1bits.T1IS
#define Timer_1_Interrupt_Enable IEC0bits.T1IE

#define Timer_2_Vector _TIMER_2_VECTOR
#define Timer_2_Interrupt_Flag IFS0bits.T2IF
#define Timer_2_Interrupt_Priority IPC2bits.T2IP
#define Timer_2_Interrupt_Subpriority IPC2bits.T2IS
#define Timer_2_Interrupt_Enable IEC0bits.T2IE

#define Timer_3_Vector _TIMER_3_VECTOR
#define Timer_3_Interrupt_Flag IFS0bits.T3IF
#define Timer_3_Interrupt_Priority IPC3bits.T3IP
#define Timer_3_Interrupt_Subpriority IPC3bits.T3IS
#define Timer_3_Interrupt_Enable IEC0bits.T3IE

#define Timer_4_Vector _TIMER_4_VECTOR
#define Timer_4_Interrupt_Flag IFS0bits.T4IF
#define Timer_4_Interrupt_Priority IPC4bits.T4IP
#define Timer_4_Interrupt_Subpriority IPC4bits.T4IS
#define Timer_4_Interrupt_Enable IEC0bits.T4IE

#define Timer_5_Vector _TIMER_5_VECTOR
#define Timer_5_Interrupt_Flag IFS0bits.T5IF
#define Timer_5_Interrupt_Priority IPC5bits.T5IP
#define Timer_5_Interrupt_Subpriority IPC5bits.T5IS
#define Timer_5_Interrupt_Enable IEC0bits.T5IE

#include "interrupt_config_Timer_Initialization.h"
#if defined(Rover)
#include "interrupt_config_Timer_PWM.h"
#include "interrupt_config_Timer_RoverMeasure.h"
#elif defined(BaseStation)
#include "interrupt_config_Timer_CN.h"
#endif

#define _Configure_Timer_Interrupts(X)  _Configure_Interrupts(X##_Timer)
#define Configure_Timer_Interrupts(...)\
 void __Configure_Interrupts(Timer) {\
	printConfigurationStarted(Timer_Interrupts);\
  FOR_EACH(_Configure_Timer_Interrupts,__VA_ARGS__)\
	printConfigurationCompleted(Timer_Interrupts);\
 }

#define _Timer_Interrupt(object, full_name, priority, subpriority, event_type, is_enabled)\
	printf("\n\r\t\t" #object "_Interrupt_Enable = " #is_enabled);\
  object##_Interrupt_Flag = false; /* Sets the initial flag state for the interrupt */\
  object##_Interrupt_Enable = is_enabled; /* Sets the enable bit for the interrupt */\
 }\
 __attribute__((vector(VECTOR(full_name)),interrupt(PRIORITY(priority)), nomips16))\
 void object##_##event_type##_Event()
#define Timer_Interrupt(object, timer_number, priority, subpriority, event_type, is_enabled)\
	_Interrupt(object##_Timer, TIMER_##timer_number, priority, subpriority)\
	_Timer_Interrupt(object##_Timer, TIMER_##timer_number, priority, subpriority, event_type, is_enabled)
	

Configure_Timer_Interrupts(Initialization
#if defined(Rover)
													,PWM
													,RoverMeasure
#elif defined(BaseStation)
													,CN
#endif
)

#endif	/* INTERRUPT_CONFIG_TIMERS_H */

