/* 
 * File:   lora_config.h
 * Author: cstgeo
 *
 * Created on March 1, 2018, 11:10 PM
 */

#ifndef LORA_CONFIG_H
#define	LORA_CONFIG_H
#include "config.h"

typedef enum { LORA_INITIALIZING,
               LORA_CONFIGURING,
               LORA_TRANSCEIVER_ENABLED,
               LORA_INITIALIZATION_ERROR,
               LORA_CONFIGURATION_ERROR
} LORA_STATE;
LORA_STATE lora_state = LORA_INITIALIZING;

typedef enum {LORA_UART_DATA_TO_SEND,/* -> */LORA_UART_SENDING_DATA,/* -> */LORA_UART_DATA_SENT,/* -> */LORA_UART_RECEIVING_STRING,/* -> */LORA_UART_STRING_RECEIVED,/* -> */LORA_UART_STRING_READ							
} LORA_UART_STATE;
LORA_UART_STATE lora_uart_state = LORA_UART_RECEIVING_STRING;

typedef enum { LORA_CONFIGURATION_NOT_CONFIGURED,
               LORA_CONFIGURATION_PAUSING_LORAWAN,
               LORA_CONFIGURATION_SETTING_RADIO_POWER,
               LORA_CONFIGURATION_SETTING_WATCHDOG_TIMER,
							 LORA_CONFIGURATION_SETTING_SYNC_WORD,
               LORA_CONFIGURATION_COMPLETE,
               LORA_CONFIGURATION_PAUSE_LORAWAN_ERROR,
               LORA_CONFIGURATION_SET_RADIO_POWER_ERROR,
               LORA_CONFIGURATION_SET_WATCHDOG_TIMER_ERROR
} LORA_CONFIGURATION_STATE;
LORA_CONFIGURATION_STATE lora_configuration_state = LORA_CONFIGURATION_NOT_CONFIGURED;

typedef enum { LORA_TRANSCEIVER_NOT_ENABLED, LORA_TRANSCEIVER_IDLE,
               LORA_TRANSCEIVER_TRANSMISSION_STARTING,/* -> */LORA_TRANSCEIVER_TRANSMITTING,
               LORA_TRANSCEIVER_RECEPTION_STARTING,/* -> */LORA_TRANSCEIVER_RECEIVING
} LORA_TRANSCEIVER_STATE;
LORA_TRANSCEIVER_STATE lora_transceiver_state = LORA_TRANSCEIVER_NOT_ENABLED;

#if defined(Rover)
#define lora_rx_time 150
#elif defined(BaseStation)
#define lora_rx_time 0
void updateCN(); // Called after transmission is sent
#endif

#define lora_pwr 20

void do_this_after_getting_lora_data();
void do_this_before_every_lora_rx();
void do_this_during_every_lora_rx();
void do_this_after_every_lora_rx();
void do_this_before_every_lora_tx();
void do_this_during_every_lora_tx();
void do_this_after_every_lora_tx();

#define lora_initialized_condition lora_received_string[0] == 'R'
#define lora_mac_paused_condition lora_received_string[0] == '4'
#define lora_radio_power_set_condition lora_received_string[0] == 'o'
#define lora_watchdog_timer_set_condition lora_received_string[0] == 'o'
#define lora_sync_word_set_condition lora_received_string[0] == 'o'
#define lora_rx_receiving_condition lora_received_string[0] == 'o'
#define lora_tx_transmitting_condition lora_received_string[0] == 'o'
#define lora_rx_received_condition lora_received_string[0] == 'r'
#define lora_tx_transmitted_condition lora_received_string[0] == 'r'
#define lora_rx_no_data_condition lora_received_string[6] == 'e'

#define print_lora_mac_pause printf("mac pause\r\n")
#define print_lora_radio_set_pwr(d) printf("radio set pwr " #d "\r\n")
#define print_lora_radio_set_wdt(d) printf("radio set wdt " #d "\r\n")
#define print_lora_radio_set_sync(s) printf("radio set sync 12" "\r\n")
#define print_lora_radio_tx(s) printf("radio tx " "%s" "\r\n", s)
#define print_lora_radio_rx(d) printf("radio rx " #d "\r\n")
#define printLoRaWaiting printf("\t...LoRa module waiting to receive...\n\r")
#define printLoRaNoData printf("\t" "...no LoRa data received yet.\n\n\r")

#define lora_mac_pause lora_configuration_state = LORA_CONFIGURATION_PAUSING_LORAWAN; PIC_to_LoRa(print_lora_mac_pause)
#define lora_radio_set_pwr(d) lora_configuration_state = LORA_CONFIGURATION_SETTING_RADIO_POWER; PIC_to_LoRa(print_lora_radio_set_pwr(d))
#define lora_radio_set_wdt(d) lora_configuration_state = LORA_CONFIGURATION_SETTING_WATCHDOG_TIMER; PIC_to_LoRa(print_lora_radio_set_wdt(d))
#define lora_radio_set_sync(s) lora_configuration_state = LORA_CONFIGURATION_SETTING_SYNC_WORD; PIC_to_LoRa(print_lora_radio_set_sync(s))
#define lora_radio_tx(s) do_this_before_every_lora_tx(); lora_transceiver_state = LORA_TRANSCEIVER_TRANSMISSION_STARTING; PIC_to_LoRa(print_lora_radio_tx(s))
#define lora_radio_rx(d) do_this_before_every_lora_rx(); lora_transceiver_state = LORA_TRANSCEIVER_RECEPTION_STARTING; PIC_to_LoRa(print_lora_radio_rx(d))

bool just_transmitted = false;
bool should_receive = false;

void configure_lora() {
	lora_uart_state = LORA_UART_RECEIVING_STRING;
	lora_configuration_state = LORA_CONFIGURATION_NOT_CONFIGURED;
	lora_transceiver_state = LORA_TRANSCEIVER_NOT_ENABLED;
	just_transmitted = false;
	should_receive = false;
}

void LoRa() {
 if (lora_transceiver_state == LORA_TRANSCEIVER_IDLE && lora_uart_state == LORA_UART_STRING_READ)
 {
	 if (just_transmitted || should_receive) {
		 lora_radio_rx(lora_rx_time);
		 just_transmitted = false;
		 should_receive = false;
	 }
	 else if (outgoing_queue.is_empty()) {
		 should_receive = true;
	 } else {
		 lora_radio_tx(outgoing_queue.nybble);
		 just_transmitted = true;
	 }
 } else
 if (lora_uart_state == LORA_UART_STRING_RECEIVED)
 {
   lora_uart_state = LORA_UART_STRING_READ;
	 if (verbose_mode) { LoRa_to_PIC(printf("%s", lora_received_string)); }
   switch (lora_transceiver_state) 
	 {
    case LORA_TRANSCEIVER_NOT_ENABLED:
		{
			switch (lora_configuration_state) {
                       case LORA_CONFIGURATION_NOT_CONFIGURED:
											 { if (lora_initialized_condition       ) { lora_mac_pause;         } break; }
                       case LORA_CONFIGURATION_PAUSING_LORAWAN:
											 { if (lora_mac_paused_condition        ) { lora_radio_set_pwr(lora_pwr); } break; }
                       case LORA_CONFIGURATION_SETTING_RADIO_POWER:
											 { if (lora_radio_power_set_condition   ) { lora_radio_set_wdt(0);  } break; }
                       case LORA_CONFIGURATION_SETTING_WATCHDOG_TIMER:
											 { if (lora_watchdog_timer_set_condition) { lora_radio_set_sync(encoded_sync_word); } break; }
                       case LORA_CONFIGURATION_SETTING_SYNC_WORD:
											 { if (lora_sync_word_set_condition) {
													lora_configuration_state = LORA_CONFIGURATION_COMPLETE;
													initialize_received_lora_data;
													printf("\t...LoRa configuration complete...\n\r");
													lora_transceiver_state = LORA_TRANSCEIVER_IDLE;
												 }	break;
											 }
                      }
		      break;
		}
    case LORA_TRANSCEIVER_TRANSMISSION_STARTING:
		{ 
			if (lora_tx_transmitting_condition) { do_this_during_every_lora_tx(); lora_transceiver_state = LORA_TRANSCEIVER_TRANSMITTING; } break; 
		}
    case LORA_TRANSCEIVER_RECEPTION_STARTING: 
		{ 
			if (lora_rx_receiving_condition) { if (verbose_mode) { printLoRaWaiting; } do_this_during_every_lora_rx(); lora_transceiver_state = LORA_TRANSCEIVER_RECEIVING; } break; 
		}
    case LORA_TRANSCEIVER_TRANSMITTING:
		{ 
			if (lora_tx_transmitted_condition) { do_this_after_every_lora_tx(); lora_transceiver_state = LORA_TRANSCEIVER_IDLE; } break; 
		}
    case LORA_TRANSCEIVER_RECEIVING:
		{
			if (lora_rx_received_condition) {
				if (lora_rx_no_data_condition) { if (verbose_mode) { printLoRaNoData; } } 
				else { do_this_after_getting_lora_data(); }  
				do_this_after_every_lora_rx(); lora_transceiver_state = LORA_TRANSCEIVER_IDLE;
				break;
			}
		}	
   }
  }
}

#endif	/* LORA_CONFIG_H */