/* File:   uart_config.h
 * Author: cstgeorg
 */

#ifndef UART_CONFIG_H
#define	UART_CONFIG_H
#include "config.h"

// UART Definitions
#define __PC_UART 3
#define __LoRa_UART 2
#define __GPS_UART 5
#define __BLE_UART 6

#define PCRXREG		U3RXREG
#define LORARXREG U2RXREG
#define GPSRXREG	U5RXREG
#define BLERXREG	U6RXREG

#define baudPC 57600
#define baudGPS 9600
#define baudLoRa 57600
#define baudBLE 115200

#define BR3 baudPC
#define BR2 baudLoRa
#define BR5 baudGPS
#define BR6 baudBLE

#define BRG3 FPB/4/BR3 - 1
#define BRG2 FPB/4/BR2 - 1
#define BRG5 FPB/4/BR5 - 1
#define BRG6 FPB/4/BR6 - 1

#if defined(Rover)
#define board_name "[Rover]"
#elif defined(BaseStation)
#define board_name "[BaseStation]"
#endif

#define print_PIC_to_LoRa printf(board_name "\t[PIC32 -> LoRa]:\t")
#define print_LoRa_to_PIC printf(board_name "\t[LoRa -> PIC32]:\t")
#define print_PIC_to_GPS printf(board_name  "\t[PIC32  -> GPS]:\t")
#define print_GPS_to_PIC printf(board_name  "\t[GPS  -> PIC32]:\t")
#define print_PIC_to_BLE printf(board_name  "\t[PIC32 -> BLE%d]:\t", E.MUX_S)
#define print_BLE_to_PIC printf(board_name  "\t[BLE%d -> PIC32]:\t", E.MUX_S)

#define set_UART_PC __XC_UART = __PC_UART
#define set_UART_LoRa __XC_UART = __LoRa_UART
#define set_UART_GPS __XC_UART = __GPS_UART
#define set_UART_BLE __XC_UART = __BLE_UART
#define PIC_to_LoRa(x) if (verbose_mode) { set_UART_PC; print_PIC_to_LoRa;	x; } set_UART_LoRa; x
#define LoRa_to_PIC(x) set_UART_PC; print_LoRa_to_PIC;	x
#define PIC_to_GPS(x)	 if (verbose_mode) { set_UART_PC; print_PIC_to_GPS;		x; } set_UART_GPS;	x
#define GPS_to_PIC(x)	 set_UART_PC; print_GPS_to_PIC;		x
#define PIC_to_BLE(x)	 if (verbose_mode) { set_UART_PC; print_PIC_to_BLE;		x; } set_UART_BLE;	x 
#define BLE_to_PIC(x)	set_UART_PC; print_BLE_to_PIC;		x

#define getc_(cc,l,u)\
	char c = u##RXREG;\
	if (c == '\n') {\
		l##_uart_state = u##_UART_STRING_RECEIVED;\
	}\
	if (c != '\0') {\
		l##_received_string[l##_received_string_index++] = c;\
		if (l##_uart_state == u##_UART_STRING_RECEIVED) {\
			l##_received_string[l##_received_string_index] = '\0';\
			l##_received_string_index = 0;\
   }\
	}

#define getc_GPS getc_(GPS,gps,GPS)
#define getc_LoRa getc_(LoRa,lora,LORA)

#define getc_BLE_1 getc_(BLE_1,ble_1,BLE_1)
#define getc_BLE_2 getc_(BLE_2,ble_2,BLE_2)
#define getc_BLE_3 getc_(BLE_3,ble_3,BLE_3)



#define printConfigurationStarted(x) printf("\n\r" "Configuring " #x "..." "\n\r")
#define printConfigurationCompleted(x) printf("\n\n\r" "..." #x " Configuration Completed." "\n\r")

#define printGeneralUARTsettings printf("\n\r\t" "PIC System Clock:\t%i MHz"\
																			 "\n\r\t\t" "PIC Peripheral Bus Clock:\t%i MHz" "\n\r", SCLK/1000000, FPB/1000000)
#define printUARTsettings(x) printf("\n\r\t" "UART " #x " Setup:"\
                                   "\n\r\t\t" "BRG:\t%i"\
                                   "\n\r\t\t" "BR:\t%i bps" "\n\r", U##x##BRG, BR##x)

/* THIS FUNCTION ASSUMES THE UART HAS ALREADY BEEN INITIALIZED */
/* Sends a character out on the UART */
void __attribute__((weak))
_mon_putc (char c)
{
  volatile unsigned int *ustatus = &U2STA;
  volatile unsigned int *umodeset = &U2MODESET;
  volatile unsigned int *ustatusset = &U2STASET;
  volatile unsigned int *txreg = &U2TXREG;
  
    if (__XC_UART == 1)
    {
      ustatus = &U1STA;
      ustatusset = &U1STASET;
      umodeset = &U1MODESET;
      txreg = &U1TXREG;
    } else 
    if (__XC_UART == 2)
    {
      ustatus = &U2STA;
      ustatusset = &U2STASET;
      umodeset = &U2MODESET;
      txreg = &U2TXREG;
    } else
    if (__XC_UART == 3)
    {
      ustatus = &U3STA;
      ustatusset = &U3STASET;
      umodeset = &U3MODESET;
      txreg = &U3TXREG;
    } else
    if (__XC_UART == 4)
    {
      ustatus = &U4STA;
      ustatusset = &U4STASET;
      umodeset = &U4MODESET;
      txreg = &U4TXREG;
    } else
    if (__XC_UART == 5)
    {
      ustatus = &U5STA;
      ustatusset = &U5STASET;
      umodeset = &U5MODESET;
      txreg = &U5TXREG;
    } else
    if (__XC_UART == 6)
    {
      ustatus = &U6STA;
      ustatusset = &U6STASET;
      umodeset = &U6MODESET;
      txreg = &U6TXREG;
    }
  
  *umodeset   = (1 << _U1MODE_UARTEN_POSITION);
  *ustatusset = (1 << _U1STA_UTXEN_POSITION);
  while ((*ustatus & (1 << _U1STA_TRMT_POSITION)) == 0);
  *txreg = c;
}

int __attribute__((weak))
_mon_getc (int canblock)
{
  int i;
  volatile unsigned int *umode = &U1MODE;
  volatile unsigned int *umodeset = &U1MODESET;
  volatile unsigned int *ustatus = &U1STA;
  volatile unsigned int *ustatusset = &U1STASET;
  volatile unsigned int *rxreg = &U1RXREG;
  volatile unsigned int *brg = &U1BRG;

    if (__XC_UART == 1)
    {
      umode = &U1MODE;
      ustatus = &U1STA;
      rxreg = &U1RXREG;
      brg = &U1BRG;
    } else
    if (__XC_UART == 2)
    {
      umode = &U2MODE;
      ustatus = &U2STA;
      rxreg = &U2RXREG;
      brg = &U2BRG;
    } else
    if (__XC_UART == 3)
    {
      umode = &U3MODE;
      ustatus = &U3STA;
      rxreg = &U3RXREG;
      brg = &U3BRG;
    } else
    if (__XC_UART == 4)
    {
      umode = &U4MODE;
      ustatus = &U4STA;
      rxreg = &U4RXREG;
      brg = &U4BRG;
    } else
    if (__XC_UART == 5)
    {
      umode = &U5MODE;
      ustatus = &U5STA;
      rxreg = &U5RXREG;
      brg = &U5BRG;
    } else
    if (__XC_UART == 6)
    {
      umode = &U6MODE;
      ustatus = &U6STA;
      rxreg = &U6RXREG;
      brg = &U6BRG;
    }
  if ((*umode & (1 << _U1MODE_UARTEN_POSITION)) == 0)
    {
      *umodeset = (1 << _U1MODE_UARTEN_POSITION);
    }
  {
    int nTimeout;

    /*
    ** Timeout is 16 cycles per 10-bit char
    */
    nTimeout = 16*10;
    while (((*ustatus & (1 << _U1STA_URXDA_POSITION)) == 0) && nTimeout) --nTimeout;
    return *rxreg;
  }

  return -1;
}

void configure_uart() {
 
 __XC_UART = __PC_UART;  
 
  /* Setup UART 3 */
		  // Enable UART
                  U3MODEbits.ON = false; // be sure it is off
                  U3MODEbits.UEN = 0b00; // RX/TX mode - nRTS/nCTS pins act as RB pins.
                  U3MODEbits.BRGH = 1; // high speed
                  U3MODEbits.PDSEL = 0b00; // 8 bit no parity
                  U3MODEbits.STSEL = 0; // one stop bit
                  U3BRG = BRG3;   //  get_pb_clock() / 4 / rate - 1; // baud rate clock
                  U3STAbits.UTXEN = true; // transmit enable
                  U3STAbits.URXEN = true; // receive enable
                  U3MODEbits.ON = true; // turn it on
                  
#if defined(Rover)
									printf("\n\r" "<>--------------[ ROVER ]--------------<>" "\n\n\r");
#elif defined(BaseStation)
									printf("\n\r" "<>----------[ BASE STATION ]-----------<>" "\n\n\r");
#endif
									if (verbose_mode) { printConfigurationStarted(UART); }
                  if (verbose_mode) { printGeneralUARTsettings; }
									if (verbose_mode) { printUARTsettings(3); }
                  
 /* Setup UART 2 */
		  // Enable UART
                  U2MODEbits.ON = false; // be sure it is off
                  U2MODEbits.UEN = 0b00; // RX/TX mode - nRTS/nCTS pins act as RB pins.
                  U2MODEbits.BRGH = 1; // high speed
                  U2MODEbits.PDSEL = 0b00; // 8 bit no parity
                  U2MODEbits.STSEL = 0; // one stop bit
                  U2BRG = BRG2;   //  get_pb_clock() / 4 / rate - 1; // baud rate clock
                  U2STAbits.UTXEN = true; // transmit enable
                  U2STAbits.URXEN = true; // receive enable
                  U2MODEbits.ON = true; // turn it on
		  
                  if (verbose_mode) { printUARTsettings(2); }
                  
#if defined(Rover)
 /* Setup UART 5 */
		  // Enable UART
                  U5MODEbits.ON = false; // be sure it is off
                  U5MODEbits.UARTEN = 1; // RX/TX mode - nRTS/nCTS pins act as RB pins.
                  U5MODEbits.BRGH = 1; // high speed
                  U5MODEbits.PDSEL = 0b00; // 8 bit no parity
                  U5MODEbits.STSEL = 0; // one stop bit
                  U5BRG = BRG5;   //  get_pb_clock() / 4 / rate - 1; // baud rate clock
                  U5STAbits.UTXEN = true; // transmit enable
                  U5STAbits.URXEN = true; // receive enable
                  U5MODEbits.ON = true; // turn it on
                  
                  if (verbose_mode) { printUARTsettings(5); }
									
/* Setup UART 6 */
		  // Enable UART
                  U6MODEbits.ON = false; // be sure it is off
                  U6MODEbits.UARTEN = 1; // RX/TX mode - nRTS/nCTS pins act as RB pins.
                  U6MODEbits.BRGH = 1; // high speed
                  U6MODEbits.PDSEL = 0b00; // 8 bit no parity
                  U6MODEbits.STSEL = 0; // one stop bit
                  U6BRG = BRG6;   //  get_pb_clock() / 4 / rate - 1; // baud rate clock
                  U6STAbits.UTXEN = true; // transmit enable
                  U6STAbits.URXEN = true; // receive enable
                  U6MODEbits.ON = true; // turn it on
                  
                  if (verbose_mode) { printUARTsettings(6); }
#endif
									if (verbose_mode) { printConfigurationCompleted(UART); }
}


#endif	/* UART_CONFIG_H */

